/**@name LIBRARY: pcConfigFGC2_LHC_PC.ctl

@author: Herve Milcent (AB-CO-IS)

Creation Date: 08/05/2007

Modification History: 
  - 09/12/2008: Herve
    new DPE: REF_RUN
    
	- 02/07/2007: Herve
		function moved to unConfigGenericFunctions.ctl
		
	- 08/06/207: Herve
		bug output address
		missing format I and V
		
	- 05/06/2007: Herve
		bug in check range
		
	- 29/05/2007: Herve
		add ST_GTWDEV
		
version 1.0

External Function : 
	. pcConfigFGC2_LHC_PC_setConfig :  set configuration of given FGC2_LHC_PC datapoint
	. pcConfigFGC2_LHC_PC_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set FGC2_LHC_PC device configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

//Constants



//@{

//------------------------------------------------------------------------------------------------------------------------

// pcConfigFGC2_LHC_PC_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

pcConfigFGC2_LHC_PC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  //IS-1911. FGC2 is outdated use FGC Generic now
  fwException_raise( exceptionInfo, "ERROR"," FG2_LHC_PC device not supported. Please import your FGC as FGC_GENERIC_PC.","");
  retun;
  
  
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "FGC2_LHC_PC", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "FGC2_LHC_PC", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigFGC2_LHC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigFGC2_LHC_PC_setConfig
/**
Purpose: set configuration of given FGC2_LHC_PC datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

pcConfigFGC2_LHC_PC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{    
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "FGC2_LHC_PC", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "FGC2_LHC_PC", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "FGC2_LHC_PC", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC2_LHC_PC_checkConfig
/**
Purpose: check data before configuration for GW_FGC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

GW_FGC_FGC2_LHC_PC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
    //IS-1911. FGC2 is outdated use FGC Generic now
  fwException_raise( exceptionInfo, "ERROR"," FG2_LHC_PC device not supported. Please import your FGC as FGC_GENERIC_PC.","");
  retun;
  
  
	bool bOk=false;
	dyn_string dsSplit, dsSplit1;
	int i, len;
	float fFloat;
	
	if (dynlen(configLine) == (PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// check format
		unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_I_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_V_FORMAT], exceptionInfo);

		// check range
		if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_I] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_I] != ""))
			unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_I], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_I], exceptionInfo);
		if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_V] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_V] != ""))
			unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_V], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_V], exceptionInfo);
				
		// check deadband
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEADBAND], "|");
		dsSplit1 = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEADBANDTYPE], "|");
		while(dynlen(dsSplit) < PC_CONFIG_FGC2_LHC_PC_INPUT_DPE)
			dynAppend(dsSplit, 0);
		while(dynlen(dsSplit1) < PC_CONFIG_FGC2_LHC_PC_INPUT_DPE)
			dynAppend(dsSplit1, 0);
		for(i=1;i<=PC_CONFIG_FGC2_LHC_PC_INPUT_DPE;i++)
			unConfigGenericFunctions_checkDeadBand(dsSplit[i], dsSplit1[i], exceptionInfo);

		// check Archive
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_ARCHIVEACTIVE], "|");
		len = dynlen(dsSplit);
		for(i=1;i<=len;i++)
			unConfigGenericFunctions_checkArchiveParameters(0, dsSplit[i], exceptionInfo);

		//check DITDT 
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DIDT], ",");
		if(dynlen(dsSplit) !=2)
			fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC2_LHC_PC_checkConfig: BAD DIDT","");
		else {
			unConfigGenericFunctions_checkFloat(dsSplit[1], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(dsSplit[2], fFloat, exceptionInfo);
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC2_LHC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC2_LHC_PC_setConfig
/**
Purpose:  set configuration of given FGC2_LHC_PC datapoint for GW_FGC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

GW_FGC_FGC2_LHC_PC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName, sAlias, sDeviceLink;
	dyn_string dsDpe, dsDescription;

	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[PC_CONFIG_FGC2_LHC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("FGC2_LHC_PC", dsDpe, dsDescription);
	unConfigGenericFunctions_setDescriptionDPE(sDpName, dsDpe, dsDescription, exceptionInfo);

// set unit
	unConfigGenericFunctions_setUnitDPE(sDpName, makeDynString(".ProcessInput.I_REF", ".ProcessInput.I_MEAS",
																															".ProcessInput.V_REF", ".ProcessInput.V_MEAS"), 
																							makeDynString(PC_CONFIG_I_UNIT, PC_CONFIG_I_UNIT,
																														PC_CONFIG_V_UNIT, PC_CONFIG_V_UNIT), exceptionInfo);

// set format
	unConfigGenericFunctions_setFormatDPE(sDpName, makeDynString(".ProcessInput.I_REF", ".ProcessInput.I_MEAS",
																																".ProcessInput.V_REF", ".ProcessInput.V_MEAS",
																																".ProcessInput.STATE_PC", ".ProcessInput.STATE_PLL", 
																																".ProcessInput.STATE_OP", ".ProcessInput.STATE_VS", 
																																".ProcessInput.ST_DCCT_A", ".ProcessInput.ST_DCCT_B", 
																																".ProcessInput.EVEN_GROUP",
																																".ProcessInput.ST_FAULTS", ".ProcessInput.ST_UNLATCHED", 
																																".ProcessInput.HEADER", ".ProcessInput.ST_WARNINGS", 
																																".ProcessInput.ST_LATCHED", ".ProcessInput.ST_MEAS_A",
																																".ProcessInput.ST_MEAS_B", ".ProcessInput.I_DIFF_MA"), 
																							makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_I_FORMAT],
																														dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_I_FORMAT], 
																														dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_V_FORMAT],
																														dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_V_FORMAT],
																														PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
																														PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
																														PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
																														PC_CONFIG_3FORMAT,
																														PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
																														PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
																														PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
																														PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT),
																							exceptionInfo);

// set CMW address config on ProcessInput DPE
	pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessInput.STATE_PC", 
																																".ProcessInput.ST_FAULTS",
																																".ProcessInput.ST_UNLATCHED",
																																".ProcessInput.HEADER",
																																".ProcessInput.ST_WARNINGS",
																																".ProcessInput.ST_LATCHED",
																																".ProcessInput.STATE_PLL",
																																".ProcessInput.STATE_OP",
																																".ProcessInput.STATE_VS",
																																".ProcessInput.ST_MEAS_A",
																																".ProcessInput.ST_MEAS_B",
																																".ProcessInput.ST_DCCT_A",
																																".ProcessInput.ST_DCCT_B",
																																".ProcessInput.EVEN_GROUP",
																																".ProcessInput.I_DIFF_MA",
																																".ProcessInput.I_REF",
																																".ProcessInput.I_MEAS",
																																".ProcessInput.V_REF",
																																".ProcessInput.V_MEAS"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC2_LHC_PC_PROPERTY,
																									makeDynBool(true, true, true, true, true, true, true, true, true, true,
																																true, true, true, true, true, true, true, true, true), 
																									exceptionInfo);
// set CMW address config ST_GTWDEV DPE
	pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessInput.ST_GTWDEV"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_PSEUDO_FGC2_LHC_PC_PROPERTY,
																									makeDynBool(true), 
																									exceptionInfo);
// set CMW address config on ProcessOutput DPE only if allowed
	if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_CMDADDRESS] == "Y") {
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.STATE_PC"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									"STATE.PC", 
																									makeDynBool(false), 
																									exceptionInfo);
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.POLL_SWITCH_STATE"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									"POL_SWITCH.STATE", 
																									makeDynBool(false), 
																									exceptionInfo);
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.REF"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									"REF", 
																									makeDynBool(false), 
																									exceptionInfo);
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.REF_RUN"),
																									"pcFGC2_LHC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEVICENAME], 
																									"REF.RUN", 
																									makeDynBool(false), 
																									exceptionInfo);
	}
	else {
		fwPeriphAddress_delete(sDpName+".ProcessOutput.STATE_PC", exceptionInfo);
		fwPeriphAddress_delete(sDpName+".ProcessOutput.POLL_SWITCH_STATE", exceptionInfo);
		fwPeriphAddress_delete(sDpName+".ProcessOutput.REF", exceptionInfo);
	}
// set smoothing on all floats
	unConfigGenericFunctions_setSmoothingDPE(sDpName, makeDynString(".ProcessInput.I_REF", 
																															".ProcessInput.I_MEAS", 
																															".ProcessInput.V_REF",
																															".ProcessInput.V_MEAS"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEADBANDTYPE], "|"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DEADBAND], "|"),
																					exceptionInfo);

// set pv_range on all floats
	unConfigGenericFunctions_setPvRangeDPE(sDpName, 
																					makeDynString(".ProcessInput.I_MEAS", 
																												".ProcessInput.V_MEAS"),
																					makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_I],
																												dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MIN_V]), 
																					makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_I],
																												dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_MAX_V]), 
																					exceptionInfo);

// set archiving 
	unConfigGenericFunctions_setArchiveDPE("FGC2_LHC_PC", sDpName, 
																					makeDynString(".ProcessInput.I_REF", 
																												".ProcessInput.I_MEAS", 
																												".ProcessInput.V_REF",
																												".ProcessInput.V_MEAS",
																												".ProcessInput.ST_FAULTS",
																												".ProcessInput.ST_UNLATCHED",
																												".ProcessInput.ST_WARNINGS",
																												".ProcessInput.ST_GTWDEV"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_ARCHIVEACTIVE], "|"), 
																					0,
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], exceptionInfo);

	// set dIdT
	dpSet(sDpName+".dIdT", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_LHC_PC_DIDT]);
}

//------------------------------------------------------------------------------------------------------------------------

//@
