/**@file

// pcConfigFGC2_PS_MPC.ctl
This library contains the function to configure the FGC2_PS_MPC.

@par Creation Date
  04/05/2010

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  H. Milcent (EN-ICE)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// pcConfigFGC2_PS_MPC_checkConfig
/** check the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
pcConfigFGC2_PS_MPC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
    //IS-1911. FGC2 is outdated use FGC Generic now
  fwException_raise( exceptionInfo, "ERROR"," FG2_PS_MPC device not supported. Please import your FGC as FGC_GENERIC_PC.","");
  retun;
  
  if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(configLine, "FGC2_PS_MPC", exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(configLine, "FGC2_PS_MPC", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","unConfigFGC2_PS_MPC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// pcConfigFGC2_PS_MPC_setConfig
/** set the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs  input, config line as dyn_string
@param exceptionInfo output, for errors
*/
pcConfigFGC2_PS_MPC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;
  
// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "FGC2_PS_MPC", exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)    // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "FGC2_PS_MPC", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0)  // Creation ok or dp already exists
    {
// 2. Generate device
// 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
  // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "FGC2_PS_MPC", exceptionInfo);
    }    
  }
}

//------------------------------------------------------------------------------------------------------------------------
// GW_FGC_FGC2_PS_MPC_checkConfig
/** check the FGC2_PS_MPC device configuration for the GW_FGC front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param configLine input, config line as dyn_string
@param exceptionInfo output, for errors
*/
GW_FGC_FGC2_PS_MPC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
  //IS-1911. FGC2 is outdated use FGC Generic now
  fwException_raise( exceptionInfo, "ERROR"," FG2_PS_MPC device not supported. Please import your FGC as FGC_GENERIC_PC.","");
  retun;
  
  bool bOk=false;
  dyn_string dsSplit, dsSplit1;
  int i, len;
  float fFloat;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC import GW_FGC_FGC2_PS_MPC", "GW_FGC_FGC2_PS_MPC_checkConfig", configLine);
  if (dynlen(configLine) == (PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_COMMON_LENGTH))
    bOk = true;
  else if (dynlen(configLine) == (PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    bOk = true;
    unConfigGenericFunctions_checkArchive(configLine[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
                                          configLine[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
                                          configLine[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
                                          exceptionInfo);
  }
  else
    bOk = false;
  
  if(bOk) {
  // check format
    unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_B_FORMAT], exceptionInfo);
    unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_I_FORMAT], exceptionInfo);
    unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_V_FORMAT], exceptionInfo);

    // check range
    if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_B] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_B] != ""))
      unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_B], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_B], exceptionInfo);
    if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_I] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_I] != ""))
      unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_I], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_I], exceptionInfo);
    if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_V] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_V] != ""))
      unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_V], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_V], exceptionInfo);
        
    // check deadband
    dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEADBAND], "|");
    dsSplit1 = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEADBANDTYPE], "|");
    while(dynlen(dsSplit) < PC_CONFIG_FGC2_PS_MPC_INPUT_DPE)
      dynAppend(dsSplit, 0);
    while(dynlen(dsSplit1) < PC_CONFIG_FGC2_PS_MPC_INPUT_DPE)
      dynAppend(dsSplit1, 0);
    for(i=1;i<=PC_CONFIG_FGC2_PS_MPC_INPUT_DPE;i++)
      unConfigGenericFunctions_checkDeadBand(dsSplit[i], dsSplit1[i], exceptionInfo);

    // check Archive
    dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_ARCHIVEACTIVE], "|");
    len = dynlen(dsSplit);
    for(i=1;i<=len;i++)
      unConfigGenericFunctions_checkArchiveParameters(0, dsSplit[i], exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC2_PS_MPC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// GW_FGC_FGC2_PS_MPC_setConfig
/** set the FGC2_PS_MPC device configuration for the GW_FGC front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
GW_FGC_FGC2_PS_MPC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sAlias, sDeviceLink;
  dyn_string dsDpe, dsDescription;

  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC import GW_FGC_FGC2_PS_MPC "+sDpName, "GW_FGC_FGC2_PS_MPC_setConfig", dsConfigs);
// if archive name are in the config->overwrite panel setting.  
  if(dynlen(dsConfigs) == (PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    if(dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

    if(dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

    if(dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[PC_CONFIG_FGC2_PS_MPC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
  }
  
// set descriptions
  unGenericDpFunctions_getDeviceDefinitionDPEDescription("FGC2_PS_MPC", dsDpe, dsDescription);
  unConfigGenericFunctions_setDescriptionDPE(sDpName, dsDpe, dsDescription, exceptionInfo);

// set unit
  unConfigGenericFunctions_setUnitDPE(sDpName, makeDynString(".ProcessInput.B_REF", ".ProcessInput.B_MEAS",
                                                             ".ProcessInput.I_MEAS",
                                                             ".ProcessInput.V_REF", ".ProcessInput.V_MEAS"), 
                                              makeDynString(PC_CONFIG_B_UNIT, PC_CONFIG_B_UNIT,
                                                            PC_CONFIG_I_UNIT,
                                                            PC_CONFIG_V_UNIT, PC_CONFIG_V_UNIT), exceptionInfo);

// set format
  unConfigGenericFunctions_setFormatDPE(sDpName, makeDynString(".ProcessInput.B_REF", 
                                                               ".ProcessInput.B_MEAS",
                                                               ".ProcessInput.I_MEAS",
                                                               ".ProcessInput.V_REF", 
                                                               ".ProcessInput.V_MEAS",
                                                               ".ProcessInput.STATE_PC", ".ProcessInput.STATE_PLL", 
                                                               ".ProcessInput.STATE_OP", ".ProcessInput.STATE_VS", 
                                                               ".ProcessInput.ST_DCCT_A", ".ProcessInput.ST_DCCT_B", 
                                                               ".ProcessInput.ST_FAULTS", ".ProcessInput.ST_UNLATCHED", 
                                                               ".ProcessInput.HEADER", ".ProcessInput.ST_WARNINGS", 
                                                               ".ProcessInput.ST_LATCHED", ".ProcessInput.ST_MEAS_A",
                                                               ".ProcessInput.ST_MEAS_B"), 
                                              makeDynString(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_B_FORMAT],
                                                            dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_B_FORMAT],
                                                            dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_I_FORMAT], 
                                                            dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_V_FORMAT],
                                                            dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_V_FORMAT],
                                                            PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
                                                            PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
                                                            PC_CONFIG_3FORMAT, PC_CONFIG_3FORMAT,
                                                            PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
                                                            PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
                                                            PC_CONFIG_5FORMAT, PC_CONFIG_5FORMAT,
                                                            PC_CONFIG_5FORMAT),
                                              exceptionInfo);

// set CMW address config on ProcessInput DPE
  pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                  sDpName, 
                                                  makeDynString(".ProcessInput.STATE_PC", 
                                                                ".ProcessInput.ST_FAULTS",
                                                                ".ProcessInput.ST_UNLATCHED",
                                                                ".ProcessInput.HEADER",
                                                                ".ProcessInput.ST_WARNINGS",
                                                                ".ProcessInput.ST_LATCHED",
                                                                ".ProcessInput.STATE_PLL",
                                                                ".ProcessInput.STATE_OP",
                                                                ".ProcessInput.STATE_VS",
                                                                ".ProcessInput.ST_MEAS_A",
                                                                ".ProcessInput.ST_MEAS_B",
                                                                ".ProcessInput.ST_DCCT_A",
                                                                ".ProcessInput.ST_DCCT_B",
                                                                ".ProcessInput.B_REF",
                                                                ".ProcessInput.B_MEAS",
                                                                ".ProcessInput.V_REF",
                                                                ".ProcessInput.V_MEAS",
                                                                ".ProcessInput.I_MEAS"),
                                                  "pcFGC2_PS_MPC", 
                                                  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                  PC_CONFIG_CMW_FGC2_PS_MPC_PROPERTY,
                                                  makeDynBool(true, true, true, true, true, true, true, true, true,
                                                                true, true, true, true, true, true, true, true, true), 
                                                  exceptionInfo);
  pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                  sDpName, 
                                                  makeDynString(".ProcessInput.PAL_CHAN_DIVPAM_ID"),
                                                  "pcFGC2_PS_MPC", 
                                                  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                  "PAL.CHAN.DIVPAM_ID", 
                                                  makeDynBool(true), 
                                                  exceptionInfo);
  pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                  sDpName, 
                                                  makeDynString(".ProcessInput.CONFIG_MODE"),
                                                  "pcFGC2_PS_MPC", 
                                                  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                  "CONFIG.MODE", 
                                                  makeDynBool(true), 
                                                  exceptionInfo);
// set CMW address config ST_GTWDEV DPE
  pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                  sDpName, 
                                                  makeDynString(".ProcessInput.ST_GTWDEV"),
                                                  "pcFGC2_PS_MPC", 
                                                  dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                  PC_CONFIG_CMW_PSEUDO_FGC2_PS_MPC_PROPERTY,
                                                  makeDynBool(true), 
                                                  exceptionInfo);
// set CMW address config on ProcessOutput DPE only if allowed
  if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_CMWOUTPUTADDRESS] == "Y") {
    pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                sDpName, 
                                                makeDynString(".ProcessOutput.PAL_CHAN_DIVPAM_ID"),
                                                "pcFGC2_PS_MPC", 
                                                dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                "PAL.CHAN.DIVPAM_ID", 
                                                makeDynBool(false), 
                                                exceptionInfo);
    pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
                                                sDpName, 
                                                makeDynString(".ProcessOutput.CONFIG_MODE"),
                                                "pcFGC2_PS_MPC", 
                                                dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEVICENAME], 
                                                "CONFIG.MODE", 
                                                makeDynBool(false), 
                                                exceptionInfo);
  }
  else {
    fwPeriphAddress_delete(sDpName+".ProcessOutput.PAL_CHAN_DIVPAM_ID", exceptionInfo);
    fwPeriphAddress_delete(sDpName+".ProcessOutput.CONFIG_MODE", exceptionInfo);
  }
// set smoothing on all floats
  unConfigGenericFunctions_setSmoothingDPE(sDpName, makeDynString(".ProcessInput.B_REF",
                                                                  ".ProcessInput.B_MEAS", 
                                                                  ".ProcessInput.V_REF",
                                                                  ".ProcessInput.V_MEAS",
                                                                  ".ProcessInput.I_MEAS"),
                                          strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEADBANDTYPE], "|"),
                                          strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_DEADBAND], "|"),
                                          exceptionInfo);

// set pv_range on all floats
  unConfigGenericFunctions_setPvRangeDPE(sDpName, 
                                          makeDynString(".ProcessInput.B_MEAS", 
                                                        ".ProcessInput.V_MEAS",
                                                        ".ProcessInput.I_MEAS"),
                                          makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_B],
                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_V],
                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MIN_I]), 
                                          makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_B],
                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_V],
                                                        dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_MAX_I]), 
                                          exceptionInfo);

// set archiving 
  unConfigGenericFunctions_setArchiveDPE("FGC2_PS_MPC", sDpName, 
                                          makeDynString(".ProcessInput.B_REF", 
                                                        ".ProcessInput.B_MEAS", 
                                                        ".ProcessInput.V_REF",
                                                        ".ProcessInput.V_MEAS",
                                                        ".ProcessInput.I_MEAS",
                                                        ".ProcessInput.ST_FAULTS",
                                                        ".ProcessInput.ST_UNLATCHED",
                                                        ".ProcessInput.ST_WARNINGS",
                                                        ".ProcessInput.ST_GTWDEV",
                                                        ".ProcessInput.PAL_CHAN_DIVPAM_ID",
                                                        ".ProcessInput.CONFIG_MODE"),
                                          strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC2_PS_MPC_ARCHIVEACTIVE], "|"), 
                                          0,
                                          dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
                                          dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
                                          dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], exceptionInfo);
}

//------------------------------------------------------------------------------------------------------------------------
// FGC2_PS_MPC_getArchiveDPE
/** return the list of FGC2_PS_MPC DPE per archive (Bool, Ana, Event)
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@return output, list of DPE
*/
dyn_string FGC2_PS_MPC_getArchiveDPE()
{
  dyn_string dsRet;
  dyn_string dsArchiveElements, exceptionInfo, dsDpel;
  dyn_int diDpelType;
  int i, len, j, k, length, lengthk, pos;
  string archiveBoolNames, archiveEventNames, archiveAnalogNames;
  string sDeviceType="FGC2_PS_MPC";
       
  // get all the DPE having an archive config from the JCOP device definition
  fwDevice_getArchiveElements(sDeviceType, dsArchiveElements, exceptionInfo);
  // get all the DPE from the device type
  unGenericDpFunctions_getDPE(sDeviceType, dsDpel, diDpelType);
  
  length = dynlen(dsArchiveElements);
  lengthk = dynlen(dsDpel);
  for(j=1;j<=length;j++)
  {
    for(k=1;k<=lengthk;k++) {
      if((sDeviceType+dsArchiveElements[j]) == dsDpel[k]) {
        switch(diDpelType[k]){
          case DPEL_BOOL:
            archiveBoolNames = archiveBoolNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_FLOAT:
            archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_INT:
          case DPEL_STRING:
          case DPEL_DYN_INT:
          case DPEL_UINT:
            archiveEventNames = archiveEventNames + dsArchiveElements[j] + ";";
            break;
        }
      }
    }
  }
  dsRet[1] = archiveBoolNames;
  dsRet[2] = archiveAnalogNames;
  dsRet[3] = archiveEventNames;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "FGC import FGC2_PS_MPC", "FGC2_PS_MPC_getArchiveDPE", dsRet);
  return dsRet;
}

//------------------------------------------------------------------------------------------------------------------------

//@}
