#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "fwGeneral/fwException.ctl"
#uses "pcConfigGenericFunctions.ctl"
#uses "pcConstant_declarations.ctl"
#uses "unConfigGenericFunctions.ctl"
#uses "unGenericDpFunctions.ctl"
#uses "unicos_declarations.ctl"

/**@name LIBRARY: pcConfigFGC_GENERIC_PC_.ctl

@author: Jean-Charles Tournier (EN-ICE-SCD)

Creation Date: 20/01/2015

Modification History: 
  - 18/04/2017: JC
    IS-1911 Initial implementation of FG_PCGENERIC
		
version 1.0

External Function : 
	. pcConfigFGC_GENERIC_PC_setConfig :  set configuration of given FGC_GENERIC datapoint
	. pcConfigFGC_GENERIC_PC_checkConfig : check data before configuration
	
Internal Functions :
	
Purpose: 
This library contains the function used to set FGC_GENERIC device configuration.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.15
	. operating system: Linux and WXP but tested only on Linux.
	. distributed system: yes.
*/

//Constants



//@{

//------------------------------------------------------------------------------------------------------------------------

// pcConfigFGC_GENERIC_PC_checkConfig
/**
Purpose: check data before configuration

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

pcConfigFGC_GENERIC_PC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	if (dynlen(configLine) >= UN_CONFIG_COMMON_LENGTH)
	{
	// 1. Common
		unConfigGenericFunctions_checkParameters(configLine, "FGC_GENERIC_PC", exceptionInfo);
	// 2. call the device check function
		unConfigGenericFunctions_checkDeviceConfig(configLine, "FGC_GENERIC_PC", exceptionInfo);
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","unConfigFGC_GENERIC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// pcConfigFGC_GENERIC_PC_setConfig
/**
Purpose: set configuration of given FGC_GENERIC_PC datapoint

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/

pcConfigFGC_GENERIC_PC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	
// 1. Initialization : checking + dpCreate
	unConfigGenericFunctions_checkAll(dsConfigs, "FGC_GENERIC_PC", exceptionInfo);
	if (dynlen(exceptionInfo) <= 0)		// Good inputs
	{
		sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
		unConfigGenericFunctions_createDp(sDpName, "FGC_GENERIC_PC", exceptionInfo);
		if (dynlen(exceptionInfo) <= 0)	// Creation ok or dp already exists
		{
// 2. Generate device
// 2.1. Parameters
			unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
	// 2.2 call the device check function
			unConfigGenericFunctions_setDeviceConfig(dsConfigs, "FGC_GENERIC_PC", exceptionInfo);
		}		
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC_GENERIC_PC_PC_checkConfig
/**
Purpose: check data before configuration for GW_FGC front-end type

Parameters:
		configLine: dyn_string, input, configs (see dyn_string indexes in constants defined)
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/

GW_FGC_FGC_GENERIC_PC_checkConfig(dyn_string configLine, dyn_string &exceptionInfo)
{
	bool bOk=false;
	dyn_string dsSplit, dsSplit1;
	int i, len;
	float fFloat;
	
	if (dynlen(configLine) == (PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH))
		bOk = true;
	else if (dynlen(configLine) == (PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		bOk = true;
		unConfigGenericFunctions_checkArchive(configLine[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
																					configLine[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
																					configLine[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
																					exceptionInfo);
	}
	else
		bOk = false;
	
	if(bOk) {
	// check format
		unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_I_FORMAT], exceptionInfo);
		unConfigGenericFunctions_checkUnitFormat("", configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_V_FORMAT], exceptionInfo);

		// check range
		if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_I] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_I] != ""))
			unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_I], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_I], exceptionInfo);
		if((configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_V] != "") && (configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_V] != ""))
			unConfigGenericFunctions_checkRange(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_V], configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_V], exceptionInfo);
				
		// check deadband
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEADBAND], "|");
		dsSplit1 = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEADBANDTYPE], "|");
		while(dynlen(dsSplit) < PC_CONFIG_FGC_GENERIC_PC_INPUT_DPE)
		{
			if (dynAppend(dsSplit, 0) == -1)
			{
				fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC_GENERIC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
				return;
			}
		}
		
		while(dynlen(dsSplit1) < PC_CONFIG_FGC_GENERIC_PC_INPUT_DPE)
		{
			if (dynAppend(dsSplit1, 0) == -1 )
			{
				fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC_GENERIC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
			}
		}
			
		for(i=1;i<=PC_CONFIG_FGC_GENERIC_PC_INPUT_DPE;i++)
			unConfigGenericFunctions_checkDeadBand(dsSplit[i], dsSplit1[i], exceptionInfo);

		// check Archive
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_ARCHIVEACTIVE], "|");
		len = dynlen(dsSplit);
        
		for(i=1;i<=len;i++)
			unConfigGenericFunctions_checkArchiveParameters("0", dsSplit[i], exceptionInfo);

		//check DITDT 
		dsSplit = strsplit(configLine[UN_CONFIG_COMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DIDT], ",");
		if(dynlen(dsSplit) !=2)
			fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC_GENERIC_PC_checkConfig: BAD DIDT","");
		else {
			unConfigGenericFunctions_checkFloat(dsSplit[1], fFloat, exceptionInfo);
			unConfigGenericFunctions_checkFloat(dsSplit[2], fFloat, exceptionInfo);
		}
	}
	else
	{
		fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC_GENERIC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
}

//------------------------------------------------------------------------------------------------------------------------

// GW_FGC_FGC_GENERIC_PC_setConfig
/**
Purpose:  set configuration of given FGC_GENERIC datapoint for GW_FGC front-end type

Parameters:
		dsConfigs: dyn_string, input, configs (see dyn_string indexes in constants defined)
		dsConfigs is the addition of the file configuration line + additional parameters needed for PVSS. These additional parameters
		are added during the import process. Their contents can be accessed using constants defined in unConfigGenericFunctions.ctl library
		
		exceptionInfo: dyn_string, output, for errors

Usage: External function

PVSS manager usage: Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: win XP.
	. distributed system: yes.
*/

GW_FGC_FGC_GENERIC_PC_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
	string sDpName;
	dyn_string dsDpe, dsDescription;

	sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];

// if archive name are in the config->overwrite panel setting.	
	if(dynlen(dsConfigs) == (PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
		if(dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];

		if(dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];

		if(dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
			dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[PC_CONFIG_FGC_GENERIC_PC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
	}
	
// set descriptions
	unGenericDpFunctions_getDeviceDefinitionDPEDescription("FGC_GENERIC_PC", dsDpe, dsDescription);
	unConfigGenericFunctions_setDescriptionDPE(sDpName, dsDpe, dsDescription, exceptionInfo);

// set unit
	unConfigGenericFunctions_setUnitDPE(sDpName, makeDynString(".ProcessInput.I_REF", ".ProcessInput.I_MEAS",
																															".ProcessInput.V_REF", ".ProcessInput.V_MEAS"), 
																							makeDynString(PC_CONFIG_I_UNIT, PC_CONFIG_I_UNIT,
																														PC_CONFIG_V_UNIT, PC_CONFIG_V_UNIT), exceptionInfo);

// set format
	unConfigGenericFunctions_setFormatDPE(sDpName, makeDynString(
																 ".ProcessInput.I_MEAS",                                  
																 ".ProcessInput.I_REF",
																 ".ProcessInput.V_MEAS",                                 
																 ".ProcessInput.V_REF"                                 
                           ), 
													makeDynString(	
                                   dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_I_FORMAT],
																  	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_I_FORMAT], 
																	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_V_FORMAT],
																	dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_V_FORMAT]																	 																	 
																	),
													exceptionInfo);

// set CMW address config on ProcessInput DPE
	pcConfigGenericFunctions_setAddressConfig(	dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
											 	sDpName, 
												makeDynString(	
                                  ".ProcessInput.STATE_PC",																 
																 ".ProcessInput.STATE_OP",
                                  ".ProcessInput.ST_FAULTS",                                 
                                  ".ProcessInput.ST_WARNINGS",
                                  ".ProcessInput.ST_UNLATCHED",
																 ".ProcessInput.I_MEAS",                                  
																 ".ProcessInput.I_REF",
																 ".ProcessInput.V_MEAS",                                 
																 ".ProcessInput.V_REF"                                 
                                 ),
												"pcFGC_GENERIC_PC", 
												dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
												PC_CONFIG_CMW_FGC_GENERIC_PC_PROPERTY,
												makeDynBool(
                               false,  //no level comparison because of internal PVSS bug ETM-1469
															false, 
															false, 
															false, 
                              false,
															true, 
															true, 
															true, 
															true 
												), 
																									exceptionInfo);
  
// set CMW address config on ProcessOutput DPE only if allowed
	if(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_CMDADDRESS] == "Y") {
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.STATE_PC"),
																									"pcFGC_GENERIC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC_GENERIC_STATE_PC_PROPERTY, 
																									makeDynBool(false), 
																									exceptionInfo);     
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.POLL_SWITCH_STATE"),
																									"pcFGC_GENERIC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC_GENERIC_STATE_POLL_SWITCH_STATE_PROPERTY, 
																									makeDynBool(false), 
																									exceptionInfo);
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.REF"),
																									"pcFGC_GENERIC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC_GENERIC_REF_PROPERTY, 
																									makeDynBool(false), 
																									exceptionInfo);
			pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.REF_RUN"),
																									"pcFGC_GENERIC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC_GENERIC_REF_RUN_PROPERTY, 
																									makeDynBool(false), 
																									exceptionInfo);
      pcConfigGenericFunctions_setAddressConfig(dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER], 
																									sDpName, 
																									makeDynString(".ProcessOutput.REG_MODE"),
																									"pcFGC_GENERIC_PC", 
																									dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEVICENAME], 
																									PC_CONFIG_CMW_FGC_GENERIC_REG_MODE_PROPERTY, 
																									makeDynBool(false), 
																									exceptionInfo);
      //IS-2001, by default REG_MODE has its address de-activated
      pcConfigGenericFunctions_disableAddress( sDpName+".ProcessOutput.REG_MODE", exceptionInfo);
	}
	else {
		fwPeriphAddress_delete(sDpName+".ProcessOutput.STATE_PC", exceptionInfo);
		fwPeriphAddress_delete(sDpName+".ProcessOutput.POLL_SWITCH_STATE", exceptionInfo);
		fwPeriphAddress_delete(sDpName+".ProcessOutput.REF", exceptionInfo);
    fwPeriphAddress_delete(sDpName+".ProcessOutput.REF_RUN", exceptionInfo);
    fwPeriphAddress_delete(sDpName+".ProcessOutput.REG_MODE", exceptionInfo);
	}
// set smoothing on all floats
	unConfigGenericFunctions_setSmoothingDPE(sDpName, makeDynString(".ProcessInput.I_REF", 
																															".ProcessInput.I_MEAS", 
																															".ProcessInput.V_REF",
																															".ProcessInput.V_MEAS"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEADBANDTYPE], "|"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DEADBAND], "|"),
																					exceptionInfo);

// set pv_range on all floats
	unConfigGenericFunctions_setPvRangeDPE(sDpName, 
																					makeDynString(".ProcessInput.I_MEAS", 
																												".ProcessInput.V_MEAS"),
																					makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_I],
																												dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MIN_V]), 
																					makeDynFloat(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_I],
																												dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_MAX_V]), 
																					exceptionInfo);

// set archiving 
	unConfigGenericFunctions_setArchiveDPE("FGC_GENERIC_PC", sDpName, 
																					makeDynString(".ProcessInput.I_REF", 
																												".ProcessInput.I_MEAS", 
																												".ProcessInput.V_REF",
																												".ProcessInput.V_MEAS",
																												".ProcessInput.ST_FAULTS",
																												".ProcessInput.ST_UNLATCHED",
																												".ProcessInput.ST_WARNINGS",
																												".ProcessInput.STATE_PC",
                                                        ".ProcessInput.STATE_OP"),
																					strsplit(dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_ARCHIVEACTIVE], "|"), 
																					0,
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], 
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG],
																					dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], exceptionInfo);

	// set dIdT
	if (dpSet(sDpName+".dIdT", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PC_CONFIG_FGC_GENERIC_PC_DIDT]) == -1)
	{
		fwException_raise(exceptionInfo,"ERROR","GW_FGC_FGC_GENERIC_PC_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
	}
  
  // Set Access control for the device - IS-2012
  string sAccessControlDomainList = UNICOS_ACCESSCONTROL_DOMAINNAME;
  string sOperator = ""; //no rights by default 
  string sExpert = ""; //no rights by default 
  
  //admin has all rights
  dyn_string dsAdmin = makeDynString(  UN_FACEPLATE_BUTTON_SELECT, UN_FACEPLATE_BUTTON_SET_VALUE, PC_FACEPLATE_BUTTON_ON, 
						                           PC_FACEPLATE_BUTTON_OFF, PC_FACEPLATE_BUTTON_IDLE, PC_FACEPLATE_BUTTON_RESET, 
						                           PC_FACEPLATE_BUTTON_POLARITY_POSITIVE, PC_FACEPLATE_BUTTON_POLARITY_NEGATIVE);
  string sAdmin = strjoin( dsAdmin, UN_ACCESS_CONTROL_SEPARATOR);

  unConfigGenericFunctions_setDeviceAccessControl(sDpName, sAccessControlDomainList, sOperator, sExpert, sAdmin); 
}

//------------------------------------------------------------------------------------------------------------------------


//@
