#uses "unGenericDpFunctions.ctl"
#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "unSelectDeselectHMI.ctl"
#uses "unGenericFunctions_core.ctl"
#uses "unGenericObject.ctl"
#uses "unConfigGenericFunctions.ctl"
/**@name LIBRARY: pcFGC_GENERIC.ctl

@author: Jean-Charles Tournier (EN-ICE-SCD)

Creation Date: 22/01/2015

Modification History:

  18/04/2017: Jean-Charles
  - Initial version


version 1.0

		
Purpose: This library contains pcFGC_GENERIC object functions.

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. pcFGC_GENERIC object (DPType etc.)
	. Global variables and $parameters defined in Analog faceplate and widget
	. PVSS version: 3.0
	. operating system: WXP.
	. distributed system: yes.
*/

// constant declaration
const int FGC_GENERIC_PC_PC_RDA_SET_TIMEOUT = 2;

//@{

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ObjectListGetValueTime
/**
Purpose: Function called from the objectList to return the time and value

Parameters:
	- sDeviceName, string, input, device name
	- sDeviceType, string, input, unicos object
	- dsReturnData, dyn_string, output, return value

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Alarm faceplate
	. PVSS version: 2.12.1 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
float fI;
int iSTATE_PC, iSTATE_OP, iST_FAULTS;
string sTime, sColor, sSTATE_PCLetter, sSTATE_OPLetter;
bool bBad, bStDevice;
mapping mSTATE_PC, mSTATE_OP;
string sSystemName, sFrontEnd, sI_MEAS;
string sFE, sAppl, sIFormat, sIUnit;
int iBitPosFGC, iWF;
	
	pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, mSTATE_PC);
	pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, mSTATE_OP);

	sSystemName = unGenericDpFunctions_getSystemName(sDeviceName);
	// get the PLC name 
	sFrontEnd = unGenericObject_GetPLCNameFromDpName(sDeviceName);
	sFrontEnd = sSystemName+sFrontEnd;

	dpGet(sDeviceName+".ProcessInput.I_MEAS", fI, 
		sDeviceName+".ProcessInput.STATE_PC", iSTATE_PC,
		sDeviceName+".ProcessInput.STATE_PC:_online.._bad", bBad,
		sDeviceName+".ProcessInput.I_MEAS:_online.._stime", sTime,
		sDeviceName+".ProcessInput.ST_FAULTS", iST_FAULTS,
		sDeviceName+".ProcessInput.STATE_OP", iSTATE_OP,
		sDeviceName+".ProcessInput.ST_GTWDEV", bStDevice);

	pcFGC_GENERIC_PC_getConfig(sDeviceName, sFE, sAppl, iBitPosFGC, iWF);
	sIFormat=dpGetFormat(sDeviceName + ".ProcessInput.I_MEAS");
	sIUnit=dpGetUnit(sDeviceName + ".ProcessInput.I_MEAS");

	sI_MEAS = strrtrim(unGenericObject_FormatValue(sIFormat, fI) + " " + sIUnit);
	
	 if(!bStDevice)
		{
		//iSTATE_PC = PC_TEXT_LABEL_FGC_UNKNOWN;
		//iSTATE_OP = PC_TEXT_LABEL_FGC_UNKNOWN;
		//sI_MEAS = PC_FGC_UNKNOWN_VALUE;
		bBad=TRUE;
		}
	
	dsReturnData[1] = sTime;
	dsReturnData[2] = "I="+sI_MEAS;
	dsReturnData[3] = bBad;
	
	if(mappingHasKey(mSTATE_PC, iSTATE_PC))
		{
		sSTATE_PCLetter = mSTATE_PC[iSTATE_PC][PC_TEXT_LABEL_TEXT];
		}
	else
		{
		sSTATE_PCLetter = PC_FGC_UNKNOWN_VALUE;
		}
	
	if(mappingHasKey(mSTATE_OP, iSTATE_OP))
		{
		sSTATE_OPLetter = mSTATE_OP[iSTATE_OP][PC_TEXT_LABEL_TEXT];
		}
	else
		{
		sSTATE_OPLetter = PC_FGC_UNKNOWN_VALUE;
		}
		
	if(!bStDevice )
		dsReturnData[4] = "NO FGC3_PC CMW DEVICE";
	else
		dsReturnData[4] = sSTATE_PCLetter+"      " +sSTATE_OPLetter;
	
	if(iST_FAULTS != 0) 
		sColor = "unFaceplate_AlarmActive";
	else
		sColor = "unSynoptic_staticText";
	
	dsReturnData[5] = sColor;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_WidgetRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in Analog widget
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction;
	dyn_string exceptionInfo;
	string sFrontEnd, sSystFE;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
	if (deviceName == "")	// In case of disconnection
	{
		deviceName = g_sDpName;
	}
	g_bUnSystemAlarmPlc = false;
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;
		
	if(sFrontEnd != "")
		{
		sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
		sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
		if(dpExists(sSystFE))
			g_bUnSystemAlarmPlc = true;
		}

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			pcFGC_GENERIC_PC_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_DPCONNECT:
			g_sDpName = deviceName;
			pcFGC_GENERIC_PC_WidgetConnect(deviceName, sSystFE, sFrontEnd);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			
			switch(g_sWidgetType)
				{
				case "SYNOPTIC": 
					pcFGC_GENERIC_PC_Synoptic_WidgetDisconnect(deviceName, sSystFE, sFrontEnd);
					break;
		
				case "FGC3_PC": 
					pcFGC_GENERIC_PC_WidgetDisconnect(deviceName, sSystFE, sFrontEnd);
					break;				
				default:
					break;				
				}
			
			pcFGC_GENERIC_PC_WidgetDisconnection(g_sWidgetType);
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_WidgetDisconnection
/**
Purpose: Animate widget disconnection

Parameters:
	- sWidgetType
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_WidgetDisconnection(string sWidgetType)
{
	switch(sWidgetType)
		{
		case "FGC_GENERIC_PC":
			setMultiValue("STATE_PC", "color", "unDataNoAccess",
										"STATE_OP", "color", "unDataNoAccess",
										"WarningText", "text", "",
										"AlarmText", "text", "",
										"WidgetArea", "visible", false,
										"SelectArea", "foreCol", "",
										"LockBmp", "visible", false,
										"I_MEAS", "foreCol", "unDataNoAccess",
						  			"V_MEAS", "foreCol", "unDataNoAccess",
						  			"BorderAlarm", "foreCol", "pcWidgetBorderOKColor");
			break;
		case "SYNOPTIC":
			setMultiValue("STATE_PC", "foreCol", "unDataNoAccess",
										"STATE_OP", "foreCol", "unDataNoAccess",
										"WarningText", "text", "",
										"I_MEAS", "foreCol", "unDataNoAccess",
						  			"I_REF", "foreCol", "unDataNoAccess");
				unGenericObject_ColoredSquareDisconnect(makeDynString("PC_PERMIT_1", "PC_PERMIT_2", "FAST_ABORT", "PWR_FAILURE", "PC_DISCH_RQ", "POST_MORTEM"));						  			
			break;
		default:
			break;	
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_WidgetConnect
/**
Purpose: Connect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_WidgetConnect(string deviceName, string sSystFE, string sFrontEnd)
{
	int iRes, iWF;
	string sSystemName;
	string sFE, sAppl;
	
//  check if the device name is of type "pcFGC_GENERIC" if not like of no PLC. -> not available.
	if(dpTypeName(deviceName) != "FGC_GENERIC_PC")
		g_bUnSystemAlarmPlc = false;
	
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	if(g_bUnSystemAlarmPlc)
		{
		pcFGC_GENERIC_PC_getConfig(deviceName, sFE, sAppl, g_iBitPosFGC, iWF);
		pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, g_mSTATE_PC);
		pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, g_mSTATE_OP);
		g_sIFormat=dpGetFormat(deviceName + ".ProcessInput.I_MEAS");
		g_sIUnit=dpGetUnit(deviceName + ".ProcessInput.I_MEAS");		

		switch(g_sWidgetType)
			{
			case "SYNOPTIC": 
				iRes = dpConnect(	"pcFGC_GENERIC_PC_Synoptic_WidgetCB", 
													deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
								 					deviceName + ".ProcessInput.ST_FAULTS",	
                          deviceName + ".ProcessInput.ST_UNLATCHED",							 				
													deviceName + ".ProcessInput.STATE_PC",
								 					deviceName + ".ProcessInput.STATE_OP",
								 					deviceName + ".ProcessInput.I_MEAS",
								 					deviceName + ".ProcessInput.I_REF",
						 							sSystFE+".alarm",
													sSystFE+".enabled");
				g_bCallbackConnected = (iRes >= 0);
				break;
	
			case "FGC_GENERIC_PC":			
				g_sVFormat=dpGetFormat(deviceName + ".ProcessInput.V_MEAS");
				g_sVUnit=dpGetUnit(deviceName + ".ProcessInput.V_MEAS");

				iRes = dpConnect(	"pcFGC_GENERIC_PC_WidgetCB", 
													deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								 					deviceName + ".statusInformation.selectedManager",
								 					deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
								 					deviceName + ".ProcessInput.ST_FAULTS",
                          deviceName + ".ProcessInput.ST_UNLATCHED",
								 					deviceName + ".ProcessInput.ST_WARNINGS",
													deviceName + ".ProcessInput.STATE_PC",
								 					deviceName + ".ProcessInput.STATE_OP",
								 					deviceName + ".ProcessInput.I_MEAS",
								 					deviceName + ".ProcessInput.V_MEAS",
						 							sSystFE+".alarm",
													sSystFE+".enabled");
				g_bCallbackConnected = (iRes >= 0);		
				break;
	
			default:
				break;
			}
		}
	else
		pcFGC_GENERIC_PC_WidgetDisconnection(g_sWidgetType);

}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_getConfig
/**
Purpose: get config

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_getConfig(string deviceName, string &sFrontEnd, string &sFEApplication, int &iBitPosFGC, int &iWFFGC)
{
	dyn_string dsSplit;
	string sFE, sAppl;
	int iDevNumber;
	int iBitPos, iWF;
	
	dsSplit = strsplit(deviceName, UN_DPNAME_SEPARATOR);
	if(dynlen(dsSplit) >= 5) { // prefix-FE-FEApplication-DevType-devNumber
		sFE = dsSplit[2]; // FE
		sAppl = dsSplit[3]; // appl
		iDevNumber = dsSplit[5]; // dev number
		iBitPos = iDevNumber/100;
		//iWF = iDevNumber - iBitPos*100;
		iWF = iBitPos;
	}
	
	sFrontEnd = sFE;
	sFEApplication = sAppl;
	iWFFGC = iWF;
	iBitPosFGC = iWF;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_loadTextLabel
/**
Purpose: load TextLabel config

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_loadTextLabel(string sTextLabel, mapping &mSTATE_PC)
{
	mapping m;
	dyn_string dsConfig, dsText, dsSplit;
	int i, len, iKey;

	if(dpExists(sTextLabel)) {
		dpGet(sTextLabel+".textList", dsText);
		len = dynlen(dsText);
		for(i=1;i<=len;i++) {
			dsSplit = strsplit(dsText[i], UN_PARAMETER_DELIMITER);
			if(dynlen(dsSplit) >1) {
				iKey = dsSplit[1];
				dsConfig[PC_TEXT_LABEL_TEXT] = dsSplit[2]; // TEXT
				if(dynlen(dsSplit) > 2)
					dsConfig[PC_TEXT_LABEL_COLOR] = dsSplit[3];
				else
					dsConfig[PC_TEXT_LABEL_COLOR] = "unSynoptic_staticText";
				m[iKey]=dsConfig;
			}
		}
	}
	mSTATE_PC = m;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_WidgetDisconnect
/**
Purpose: Disconnect function for the widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_WidgetDisconnect(string deviceName, string sSystFE, string sFrontEnd)
{
	int iRes;

	if(g_bUnSystemAlarmPlc)
	{
		iRes = dpDisconnect("pcFGC_GENERIC_PC_WidgetCB", 
													deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								 					deviceName + ".statusInformation.selectedManager",
								 					deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
								 					deviceName + ".ProcessInput.ST_FAULTS",
                          deviceName + ".ProcessInput.ST_UNLATCHED",
								 					deviceName + ".ProcessInput.ST_WARNINGS",
													deviceName + ".ProcessInput.STATE_PC",
								 					deviceName + ".ProcessInput.STATE_OP",
								 					deviceName + ".ProcessInput.I_MEAS",
								 					deviceName + ".ProcessInput.V_MEAS",
						 							sSystFE+".alarm",
													sSystFE+".enabled"
												);
		g_bCallbackConnected = !(iRes >= 0);
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_Synoptic_WidgetDisconnect
/**
Purpose: Disconnect function for the Synoptic widget data

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in Analog faceplate
	. PVSS version: 3.0 
	. operating system: XP
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_Synoptic_WidgetDisconnect(string deviceName, string sSystFE, string sFrontEnd)
{
int iRes;

	if(g_bUnSystemAlarmPlc)
		{
		iRes = dpDisconnect(	"pcFGC_GENERIC_PC_Synoptic_WidgetCB", 
													deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
								 					deviceName + ".ProcessInput.ST_FAULTS",
                          deviceName + ".ProcessInput.ST_UNLATCHED",
													deviceName + ".ProcessInput.STATE_PC",
								 					deviceName + ".ProcessInput.STATE_OP",
								 					deviceName + ".ProcessInput.I_MEAS",
								 					deviceName + ".ProcessInput.I_REF",
						 							sSystFE+".alarm",
													sSystFE+".enabled");
		g_bCallbackConnected = !(iRes >= 0);
		}
}	

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_WidgetCB
/**
Purpose: Intermediate function for widget animation

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
	* 
	* deviceName + ".statusInformation.selectedManager:_lock._original._locked",
								 					deviceName + ".statusInformation.selectedManager",
								 					deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
								 					deviceName + ".ProcessInput.ST_FAULTS",
                          deviceName + ".ProcessInput.ST_UNLATCHED",
								 					deviceName + ".ProcessInput.ST_WARNINGS",
													deviceName + ".ProcessInput.STATE_PC",
								 					deviceName + ".ProcessInput.STATE_OP",
								 					deviceName + ".ProcessInput.I_MEAS",
								 					deviceName + ".ProcessInput.V_MEAS",
						 							sSystFE+".alarm",
													sSystFE+".enabled"
	* 
*/
pcFGC_GENERIC_PC_WidgetCB(string sDpLocked, bool bLocked,
										string sDpSelectedManager, string sSelectedManager,
										string sDpInvalid, bool bInvalid,
										string sDpST_FAULTS, string sST_FAULTS,
										string sDpST_UNLATCHED, string sST_UNLATCHED,                    
										string sDpST_WARNINGS, string sST_WARNINGS,
										string sDpSTATE_PC, string sSTATE_PC,
										string sDpSTATE_OP, string sSTATE_OP,
										string sDpI_MEAS, float fI_MEAS,
										string sDpV_MEAS, float fV_MEAS,
										string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
										string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled)
{
  string selectColor, sAlarmColor, sWarningLetter, sAlarmLetter, sWarningColor, sSTATE_PCColor, sSTATE_OPColor, sSTATE_PCLetter, sSTATE_OPLetter;
  bool bSelectVisible, bInvalidData=false;
  string sI_MEAS, sMEASColor, sV_MEAS;
  string sBodyColor;
	
	if(g_bSystemConnected)
		{
		unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bSelectVisible);
		
		sI_MEAS = strrtrim(unGenericObject_FormatValue(g_sIFormat, fI_MEAS) + " " + g_sIUnit);
		sV_MEAS = strrtrim(unGenericObject_FormatValue(g_sVFormat, fV_MEAS) + " " + g_sVUnit);
		
		if(strlen(sST_WARNINGS) > 0)
			{
			sAlarmLetter = "W";
			sAlarmColor = "unFaceplate_AlarmActive";
			}
			
		if(strlen(sST_FAULTS) > 0)
			sBodyColor = "unFaceplate_AlarmActive";
		else
			sBodyColor = "pcWidgetBorderOKColor";
			
		if (bInvalid)
			{
			sWarningLetter = UN_WIDGET_TEXT_INVALID;
			bInvalidData = TRUE;
			}	
		else if (!bSystemIntegrityAlarmEnabled || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
			{
			bInvalidData = TRUE;
			sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
			}
		
		if (bInvalidData)
			{
			sWarningColor = "unDataNotValid";			
			sMEASColor = "unDataNotValid";			
			}
		else
			{
			//sMEASColor = "unWidget_ControlStateAuto";	
			sMEASColor = "unDisplayValue_Status";			
			sWarningLetter = "";
			}		
		
		// get STATE_PC, STATE_OP, 
		sSTATE_PCLetter = sSTATE_PC;
		sSTATE_OPLetter = sSTATE_OP;
		sSTATE_OPColor = bInvalidData == true ? "unDataNotValid" : "unDisplayValue_Status";
    sSTATE_PCColor = _pcFGC_GENERIC_PC_getStatePcColor(bInvalidData, sSTATE_PC);

		setMultiValue(	"WarningText", "text", sWarningLetter, 
						"WarningText", "foreCol", sWarningColor,
						"AlarmText", "text", sAlarmLetter, 
						"AlarmText", "foreCol", sAlarmColor,
						"STATE_PC", "text", cutText(sSTATE_PCLetter, 18),
						"STATE_PC", "toolTipText", sSTATE_PCLetter,            
						"STATE_PC", "foreCol", sSTATE_PCColor,
						"STATE_OP", "text", cutText(sSTATE_OPLetter, 18), 
						"STATE_OP", "toolTipText", sSTATE_OPLetter,            
						"STATE_OP", "foreCol", sSTATE_OPColor,
						"SelectArea", "foreCol", selectColor,
						"WidgetArea", "visible", true,
						"LockBmp", "visible", bSelectVisible,
						"I_MEAS", "text", sI_MEAS,
						"I_MEAS", "foreCol", sMEASColor,
						"V_MEAS", "text", sV_MEAS,
						"V_MEAS", "foreCol", sMEASColor,
						"BorderAlarm", "foreCol", sBodyColor);
    
		}
}

/**
  Simple function for the display of PC and OP state in the widget
  It takes a text as input and a max lentgh. If the text is smaller than
  max length it returns the text otherwise it returns the text truncated
  and concatanated with dots: e.g
  cutText("abcdef", 6) returns "abcdef"
  cutText("abcdef", 4) returns "abc..."
  **/
private string cutText( string sText, int iMaxCharacters)
{
  string sCutText;

  //easy case, nothing to do
  if( strlen(sText) <= iMaxCharacters)
  {
    return sText;
  }  
  
  //Now the text is longer and we need to cut it
  string sSuffixText = "...";
  
  sCutText = substr( sText, 0, iMaxCharacters - strlen(sSuffixText));
  
  sCutText += sSuffixText;
  
  return sCutText;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_Synoptic_WidgetCB
/**
Purpose: Intermediate function for Synoptic widget animation

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: XP
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_Synoptic_WidgetCB(	string sDpInvalid, bool bInvalid,
																	string sDpST_FAULTS, string sST_FAULTS,
                                  string sDpST_UNLATCHED, string sST_UNLATCHED,
																	string sDpSTATE_PC, string sSTATE_PC,
																	string sDpSTATE_OP, string sSTATE_OP,
																	string sDpI_MEAS, float fI_MEAS,
																	string sDpI_REF, float fI_REF,
																	string sDpSystemIntegrityAlarmValue, int iSystemIntegrityAlarmValue, 
																	string sDpSystemIntegrityAlarmEnabled, bool bSystemIntegrityAlarmEnabled)
{
string sSTATE_PCColor, sSTATE_OPColor, sSTATE_PCLetter, sSTATE_OPLetter;
string sI_MEAS, sI_REF, sBadColor, sOkColor, sMEASColor, sWarningLetter, sWarningColor;
bool bInvalidData, bB2;
	
	if(g_bSystemConnected)
		{
		
		sI_MEAS = strrtrim(unGenericObject_FormatValue(g_sIFormat, fI_MEAS) + " " + g_sIUnit);
		sI_REF = strrtrim(unGenericObject_FormatValue(g_sIFormat, fI_REF) + " " + g_sIUnit);
						
	 if (bInvalid)
			{
			sWarningLetter = UN_WIDGET_TEXT_INVALID;
			bInvalidData = bInvalid;			
			}
		else if(!bSystemIntegrityAlarmEnabled || (iSystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
			{
			bInvalidData = TRUE;
			sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
			}
			
		if (bInvalidData)
			{
			sOkColor="unDataNotValid";
			sBadColor="unDataNotValid";			
			sSTATE_PCColor="unDataNotValid";
			sSTATE_OPColor="unDataNotValid";
			sMEASColor="unDataNotValid";
			sWarningColor = "unDataNotValid";				
			}
		else
			{
			sBadColor = "red";
			sOkColor = "green";
			sMEASColor = "unDisplayValue_Status";			
			sWarningLetter = "";
			}
	
		// get STATE_PC, STATE_OP		
		sSTATE_PCLetter = sSTATE_PC;
		sSTATE_OPLetter = sSTATE_OP;
		sSTATE_PCColor = bInvalidData == true ? "unDataNotValid" : "unDisplayValue_Status";
		sSTATE_OPColor = bInvalidData == true ? "unDataNotValid" : "unDisplayValue_Status";
    sSTATE_PCColor = _pcFGC_GENERIC_PC_getStatePcColor(bInvalidData, sSTATE_PC);
    
    dyn_string dsST_UNLATCHED = strsplit(sST_UNLATCHED, " ");
    dyn_string dsST_FAULT = strsplit(sST_FAULTS, " ");
    
    unGenericObject_ColoredSquareAnimate("POST_MORTEM", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED,"POST_MORTEM"), FALSE, sOkColor, sBadColor, bInvalidData);


    // g_sInterlockType is a static property of the FGC stored in the description of .ProcessInput at field [5]
		switch (g_sInterlockType)
			{
			case "A":        
				unGenericObject_ColoredSquareAnimate("PWR_FAILURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PWR_FAILURE"), FALSE, sOkColor, sBadColor, bInvalidData);				
				unGenericObject_ColoredSquareAnimate("PC_PERMIT_1", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
				unGenericObject_ColoredSquareAnimate("FAST_ABORT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_FAULT, "FAST_ABORT"), FALSE, sOkColor, sBadColor, bInvalidData);
				unGenericObject_ColoredSquareAnimate("PC_DISCH_RQ", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_DISCH_RQ"), FALSE, sOkColor, sBadColor, bInvalidData);		
				break;
				
			case "B1":
				unGenericObject_ColoredSquareAnimate("PWR_FAILURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PWR_FAILURE"), FALSE, sOkColor, sBadColor, bInvalidData);		
				unGenericObject_ColoredSquareAnimate("PC_PERMIT_1", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
				unGenericObject_ColoredSquareAnimate("FAST_ABORT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_FAULT, "FAST_ABORT"), FALSE, sOkColor, sBadColor, bInvalidData);
				break;
							
			case "B2":				
				unGenericObject_ColoredSquareAnimate("PWR_FAILURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PWR_FAILURE"), FALSE, sOkColor, sBadColor, bInvalidData);		
				if(g_bB2)
					unGenericObject_ColoredSquareAnimate("PC_PERMIT_2", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
				else
					unGenericObject_ColoredSquareAnimate("PC_PERMIT_1", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
				unGenericObject_ColoredSquareAnimate("FAST_ABORT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_FAULT, "FAST_ABORT"), FALSE, sOkColor, sBadColor, bInvalidData);
				break;			
			
			case "C":				
				unGenericObject_ColoredSquareAnimate("PWR_FAILURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PWR_FAILURE"), FALSE, sOkColor, sBadColor, bInvalidData);		
				unGenericObject_ColoredSquareAnimate("PC_PERMIT_1", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
			break;

			case "D":				
				unGenericObject_ColoredSquareAnimate("PC_PERMIT_1", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), TRUE, sOkColor, sBadColor, bInvalidData);
			break;
			
			default:
				break;			
			}
    
   
		

		setMultiValue(	"WarningText", "text", sWarningLetter, 
										"WarningText", "foreCol", sWarningColor,
										"STATE_PC", "text", sSTATE_PCLetter, 
										"STATE_PC", "foreCol", sSTATE_PCColor,
										"STATE_OP", "text", sSTATE_OPLetter, 
										"STATE_OP", "foreCol", sSTATE_OPColor,
						  			"I_MEAS", "text", sI_MEAS,
						  			"I_MEAS", "foreCol", sMEASColor,
						  			"I_REF", "text", sI_REF,
						  			"I_REF", "foreCol", sMEASColor);
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_MenuConfiguration
/**
Purpose: configuration of popup menu

Parameters:
	- sDpName, string, input, device name
	- sDpType, string, input, unicos object
	- dsAccessOk, dyn_string, input, authorized action
	- menuList, dyn_string, output, menu to pop-up
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
	dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);
	bool bAddress=pcFGC_GENERIC_PC_checkAddresOutput(sDpName);
	if(bAddress)
		unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
	unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_checkAddresOutput
/**
Purpose: check if the output DPE have an address config

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
bool pcFGC_GENERIC_PC_checkAddresOutput(string sDpName)
{
	bool bExist, bActive;
	dyn_anytype daConfig;
	dyn_string exInfo;
	
	fwPeriphAddress_get(sDpName+".ProcessOutput.STATE_PC", bExist, daConfig, bActive, exInfo);
	return bExist;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_HandleMenu
/**
Purpose: handle the answer of the popup menu

Parameters:
	- deviceName, string, input, the device dp name
	- sDpType, string, input, the device dp type
	- menuList, dyn_string, input, list of requested action
	- menuAnswer, int, input, selected action from the menuList
	
Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. Constants UN_POPUPMENU_****
	. PVSS version: 3.0 
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int menuAnswer)
{
	dyn_string exceptionInfo;

	unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, menuAnswer, exceptionInfo);

	if (dynlen(exceptionInfo) > 0)
		unSendMessage_toExpertException("Right Click",exceptionInfo);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ButtonUserAccess
/**
Purpose: User access for FGC3_PC buttons

Parameters: 
	- sDpName, string, input, device name
	- sDpType, string, input, dptype
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
	bool operator, expert, admin;
	int i, length;
	dyn_bool dbPermissions;
	dyn_string exceptionInfo;
  string sOperAction, sExpAction, sAdminAction;
  bool bActionDefined = false;

  
  unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
    
  if ( (sOperAction   != UNICOS_ACCESSCONTROL_DOMAINNAME) || 
       (sExpAction    != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
       (sAdminAction  != UNICOS_ACCESSCONTROL_DOMAINNAME)
    )
  {
    bActionDefined=true;
  }
  
	operator = dbPermissions[2];
	expert = dbPermissions[3];
	admin = dbPermissions[4];
	dsAccess = makeDynString();
  
	if (operator)
	{
    if(bActionDefined)
    {
      dynAppend(dsAccess, strsplit(sOperAction, UN_ACCESS_CONTROL_SEPARATOR));
    }
  }
  
	if (expert)
	{
    if(bActionDefined)
    {
      dynAppend(dsAccess, strsplit(sExpAction, UN_ACCESS_CONTROL_SEPARATOR));
    }
  }
  
	if(admin)
  {
    if(bActionDefined)
    {
      dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
    }
  }


}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ButtonRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in FGC2_LHC_PC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ButtonRegisterCB(string sDpSystemConnected, bool bSystemConnected)
{
	string deviceName, sSystemName, sDeviceType, sFunction;
	int iAction, iRes, iManReg01;
	dyn_string dsFunctions, exceptionInfo, dsUserAccess;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	
	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			pcFGC_GENERIC_PC_ButtonDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
// call the DeviceType_ButtonUserAccess to get the list of allowed buttons.
//			DeviceType_ButtonUserAccess(deviceName, deviceType, g_dsUserAccess);

			sDeviceType=dpTypeName(deviceName);
			unGenericObject_GetFunctions(sDeviceType, dsFunctions, exceptionInfo);
			sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
						 
			if (sFunction != "")
			{       
				evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
							 "dyn_string dsUserAccessReturn;" +
							 "if (isFunctionDefined(\"" + sFunction  + "\"))" +
							 "    {" +
					 			sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
							 "    }" +
							 "return dsUserAccessReturn; }", makeDynString(), deviceName, sDeviceType);
						
				g_dsUserAccess=dsUserAccess;	
			}
			else
				fwException_raise(exceptionInfo,"ERROR", "pcFGC_GENERIC_PC_ButtonRegisterCB: " + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

			g_bAddress=pcFGC_GENERIC_PC_checkAddresOutput($sDpName);
			g_bOnOff=isFunctionDefined("FGC_GENERIC_PC_OffCommand");
			iRes = dpConnect("pcFGC_GENERIC_PC_ButtonAnimationCB",
									 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
									 				 deviceName + ".statusInformation.selectedManager",
									 				 deviceName + ".ProcessInput.STATE_PC");
			g_bCallbackConnected = (iRes >= 0);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			iRes = dpDisconnect("pcFGC_GENERIC_PC_ButtonAnimationCB",
									 				 deviceName + ".statusInformation.selectedManager:_lock._original._locked",
									 				 deviceName + ".statusInformation.selectedManager",
									 				 deviceName + ".ProcessInput.STATE_PC");
				g_bCallbackConnected = !(iRes >= 0);
			pcFGC_GENERIC_PC_ButtonDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ButtonAnimationCB
/**
Purpose: animate buttons

Parameters: 
	- dbPermissions, dyn_bool, input, list of permissions, length = 4, order : 1=monitor, 2=operator, 3=expert, 4=admin
	- dsAccess, dyn_string, output, list of allowed buttons
	
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager,
																				string sDpSTATE_PC, string sSTATE_PC)
{
	pcFGC_GENERIC_PC_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "", g_dsUserAccess, 
																makeDynString(bLocked, sSelectedManager, g_bAddress, sSTATE_PC, g_bOnOff, ""));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ButtonSetState
/**
Purpose: set button state

Parameters: 
	- deviceName, string, input, device
	- sDpType, string, input, dptype
	- dsUserAccess, dyn_string, input, user access
	- dsData, dyn_string, parameter
		
Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
	bool bSelected, bAddress, bOnOff, bOffWorking, bOnWorking;
	string localManager;
	bool bLocked, ListEnabled;
	string sSelectedManager;
	string sSTATE_PC;
	dyn_bool dbState;
	
	bLocked = (bool)dsData[1];
	sSelectedManager = dsData[2];
	bAddress = (bool)dsData[3];
	sSTATE_PC = dsData[4]; 
	bOnOff = (bool)dsData[5];
	
  // 1. Selection state
	localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
	bSelected = (localManager == "S");

  //DebugN(myModuleName(), myPanelName());
  // check if contextuel button in unicosHMI: yes --> do not show the buttons
	if(myPanelName() == "contextualButton.pnl;"+deviceName)
  {
		bAddress = false;
  }
  
 	//3. Button Select state
	if(bAddress) 
  {
    // set all buttons state and set text
		pcFGC_GENERIC_PC_getButtonState(dbState, sSTATE_PC, dsUserAccess);
		bOnWorking = g_bOnRunning.text;
		if(bOnOff) {
			bOffWorking = g_bOffRunning.text;
			setMultiValue("buttonOff", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_OFFPOS]&!bOffWorking&!bOnWorking, 
											"buttonOff", "text", PC_FACEPLATE_BUTTON_OFF);
    }
	  else
    { 
      setMultiValue("buttonOff", "enabled", false, "buttonOff", "text", "");
    }
        
    setMultiValue(  "buttonOn", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_ONPOS]&!bOffWorking&!bOnWorking, 
										"buttonSetValue", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_SETVALUEPOS]&!bOffWorking&!bOnWorking,
                    "buttonIdle", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_IDELPOS]&!bOffWorking&!bOnWorking,                    
										"buttonChangePolarityPositive", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_POLARITY_POSITIVEPOS]&!bOffWorking&!bOnWorking,
										"buttonChangePolarityNegative", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_POLARITY_NEGATIVEPOS]&!bOffWorking&!bOnWorking,
										"buttonReset", "enabled", bSelected&dbState[PC_FACEPLATE_BUTTON_RESETPOS]&!bOffWorking&!bOnWorking,
									  "buttonOn", "text", PC_FACEPLATE_BUTTON_ON,  
										"buttonSetValue", "text", UN_FACEPLATE_BUTTON_SET_VALUE,
										"buttonIdle", "text", PC_FACEPLATE_BUTTON_IDLE, 
										"buttonChangePolarityPositive", "text", PC_FACEPLATE_BUTTON_POLARITY_POSITIVE,
										"buttonChangePolarityNegative", "text", PC_FACEPLATE_BUTTON_POLARITY_NEGATIVE,
										"buttonReset", "text", PC_FACEPLATE_BUTTON_RESET, "buttonSelect", "text", UN_FACEPLATE_BUTTON_SELECT);
		unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));	

	}
	else 
  {
    // disable all buttons, set text to ""
		setMultiValue("buttonOn", "enabled", false, "buttonOff", "enabled", false, "buttonSetValue", "enabled", false,
										"buttonIdle", "enabled", false, "buttonChangePolarityPositive", "enabled", false, "buttonChangePolarityNegative", "enabled", false,
										"buttonReset", "enabled", false, "buttonSelect", "enabled", false,
									"buttonOn", "text", "", "buttonOff", "text", "", "buttonSetValue", "text", "",
										"buttonIdle", "text", "", "buttonChangePolarityPositive", "text", "",
										"buttonReset", "text", "", "buttonSelect", "text", "", "buttonChangePolarityPositive", "text", "");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_getButtonState
/**
Purpose: get the state for all the button. 
  
  c.f. IS-2021 for the matrix of button enabled definition

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/
pcFGC_GENERIC_PC_getButtonState(dyn_bool &dbState, string sSTATE_PC, dyn_string dsAccess)
{
	dbState[PC_FACEPLATE_BUTTON_ONPOS] = (sSTATE_PC == PC_FGC_STATE_OFF) 
                                       && (dynContains(dsAccess, PC_FACEPLATE_BUTTON_ON) > 0);

  dbState[PC_FACEPLATE_BUTTON_POLARITY_POSITIVEPOS] = (sSTATE_PC == PC_FGC_STATE_OFF) 
                                                      && (dynContains(dsAccess, PC_FACEPLATE_BUTTON_POLARITY_POSITIVE) > 0);
  
	dbState[PC_FACEPLATE_BUTTON_POLARITY_NEGATIVEPOS] = (sSTATE_PC == PC_FGC_STATE_OFF) 
                                                      && (dynContains(dsAccess, PC_FACEPLATE_BUTTON_POLARITY_NEGATIVE) > 0);  
  
	dbState[PC_FACEPLATE_BUTTON_SETVALUEPOS] = (sSTATE_PC == PC_FGC_STATE_IL) 
                                             && (dynContains(dsAccess, UN_FACEPLATE_BUTTON_SET_VALUE) > 0);
  
	dbState[PC_FACEPLATE_BUTTON_IDELPOS] = ((sSTATE_PC == PC_FGC_STATE_OFF) || (sSTATE_PC == PC_FGC_STATE_SB) || (sSTATE_PC == PC_FGC_STATE_RUNNING))
																						 && (dynContains(dsAccess, PC_FACEPLATE_BUTTON_IDLE) > 0);
  
	dbState[PC_FACEPLATE_BUTTON_RESETPOS] = (sSTATE_PC == PC_FGC_STATE_FLT_OFF) 
																						 && (dynContains(dsAccess, PC_FACEPLATE_BUTTON_RESET) > 0);

	

}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_UserLoginGetButtonState
/**
Purpose: function called before the button set state when a user logs in. 

Parameters:
	deviceName: string, input, the device name
	dsReturnData: dyn_string, output, the data are returned here

Usage: Internal function

PVSS manager usage:  Ctrl, NG, NV

Constraints:
	. This library works with an unicos button panel and need global variables declared in this panel
		-> g_dsUserAccess, dyn_string, button identifier is added to this variable if user access 
		-> $sDpName, string, the device DP name
		-> g_sFaceplateButtonType, string, faceplate type
	. PVSS version: 3.0 
	. operating system: NT, W2000 and Linux, but tested only under W2000.
	. distributed system: yes but not tested.
*/
pcFGC_GENERIC_PC_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
	int iRes;
	string sSTATE_PC;
	string sDpLock, sDpSelectedManager, sSelectedManager, sSystemName;
	bool bLocked, bAddress=pcFGC_GENERIC_PC_checkAddresOutput(deviceName), bOnOff=isFunctionDefined("FGC_GENERIC_PC_OffCommand");

// 2. Get parameters to call the set button function
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);

	sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
	sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
	
	if (dpExists(deviceName))
	{
		iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager, deviceName+".ProcessInput.STATE_PC", sSTATE_PC);
	}
	else
	{
		iRes = -1;
	}
	if (iRes < 0) // Errors during dpget
	{
		bLocked = false;
		sSelectedManager = "";
	}
	dsReturnData=makeDynString((string)bLocked, sSelectedManager, bAddress, sSTATE_PC, bOnOff, "");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_ButtonDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_ButtonDisconnection()
{
	setMultiValue("buttonOn", "enabled", false, "buttonOff", "enabled", false, "buttonSetValue", "enabled", false,
									"buttonIdle", "enabled", false, "buttonChangePolarity", "enabled", false,
									"buttonReset", "enabled", false, "buttonSelect", "enabled", false);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateStatusRegisterCB
/**
Purpose: register callback function

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
	string deviceName, sSystemName;
	int iAction, iRes;
	dyn_string exceptionInfo;
	bool bRemote;
	
	deviceName = unGenericDpFunctions_getDpName($sDpName);
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
	unDistributedControl_isRemote(bRemote, sSystemName);
	if(bRemote)
		g_bSystemConnected = bSystemConnected;
	else
		g_bSystemConnected = true;

	unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
	switch (iAction)
		{
		case UN_ACTION_DISCONNECT:
			pcFGC_GENERIC_PC_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_DPCONNECT:
			pcFGC_GENERIC_PC_FaceplateConnect(deviceName);
			break;
		case UN_ACTION_DPDISCONNECT_DISCONNECT:
			pcFGC_GENERIC_PC_FaceplateDisconnect(deviceName);
			pcFGC_GENERIC_PC_FaceplateStatusDisconnection();
			break;
		case UN_ACTION_NOTHING:
		default:
			break;
		}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateConnect
/**
Purpose: connect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateConnect(string deviceName)
{
	int iRes, iWF;
	dyn_string exceptionInfo;
	string sFrontEnd, sSystFE, sSystemName, sFE, sAppl;
	string sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

	g_bUnSystemAlarmPlc = false;
// get the systemName
	sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
	sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
	g_sFrontEndDp = sFrontEnd;
	
	if(sFrontEnd != ""){
		sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
		sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
		if(dpExists(sSystFE)) {
			g_bUnSystemAlarmPlc = true;
		}
	}
		
	if(g_bUnSystemAlarmPlc) {
		pcFGC_GENERIC_PC_getConfig(deviceName, sFE, sAppl, g_iBitPosFGC, iWF);
		pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_PC_TEXT_LABELDP, g_mSTATE_PC);
		pcFGC_GENERIC_PC_loadTextLabel(PC_STATE_OP_TEXT_LABELDP, g_mSTATE_OP);
		g_sIFormat 		= dpGetFormat(deviceName + ".ProcessInput.I_MEAS");
		g_sIUnit 			= dpGetUnit(deviceName + ".ProcessInput.I_MEAS");
		g_sVFormat 		= dpGetFormat(deviceName + ".ProcessInput.V_MEAS");
		g_sVUnit 			= dpGetUnit(deviceName + ".ProcessInput.V_MEAS");
		
		iRes = dpConnect("pcFGC_GENERIC_PC_FaceplateStatusAnimationCB",
											deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
							 				deviceName + ".ProcessInput.STATE_PC",
							 				deviceName + ".ProcessInput.STATE_OP",
							 				deviceName + ".ProcessInput.ST_FAULTS", 
                      deviceName + ".ProcessInput.ST_UNLATCHED",
							 				deviceName + ".ProcessInput.ST_WARNINGS", 
						 					deviceName + ".ProcessInput.I_MEAS",
						 					deviceName + ".ProcessInput.V_MEAS",
						 					deviceName + ".ProcessInput.I_REF",
						 					deviceName + ".ProcessInput.V_REF",
				 							sSystFE+".alarm",
											sSystFE+".enabled"
											);
																												
	}
	else
		pcFGC_GENERIC_PC_FaceplateStatusDisconnection();
	g_bCallbackConnected = (iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateDisconnect
/**
Purpose: disconnect function for faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables and $parameters defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateDisconnect(string deviceName)
{
	int iRes;
	string sFrontEnd, sSystFE, sSystemName;

	if(g_bUnSystemAlarmPlc) {
// get the systemName
		sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
		sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

		sSystFE = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
		sFrontEnd = sSystemName+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);

		iRes = dpDisconnect("pcFGC_GENERIC_PC_FaceplateStatusAnimationCB",
											deviceName + ".ProcessInput.STATE_PC:_online.._invalid",
							 				deviceName + ".ProcessInput.STATE_PC",
							 				deviceName + ".ProcessInput.STATE_OP",
							 				deviceName + ".ProcessInput.ST_FAULTS", 
                      deviceName + ".ProcessInput.ST_UNLATCHED",                      
							 				deviceName + ".ProcessInput.ST_WARNINGS",  
						 					deviceName + ".ProcessInput.I_MEAS",
						 					deviceName + ".ProcessInput.V_MEAS",
						 					deviceName + ".ProcessInput.I_REF",
						 					deviceName + ".ProcessInput.V_REF",
				 							sSystFE+".alarm",
											sSystFE+".enabled"
											);
	}
	g_bCallbackConnected = !(iRes >= 0);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateStatusAnimationCB
/**
Purpose: Animate faceplate

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateStatusAnimationCB(string sDpInvalid, bool bInvalid,
										string sDp_STATE_PC, string sSTATE_PC,	
										string sDp_STATE_OP, string sSTATE_OP,
										string sDp_ST_FAULTS, string sST_FAULTS,
                    string sDp_ST_UNLACTHED, string sST_UNLATCHED,
										string sDp_ST_WARNINGS, string sST_WARNINGS,
										string sDp_I_MEAS, float fI_MEAS,
										string sDp_V_MEAS, float fV_MEAS,
										string sDp_I_REF, float fI_REF,
										string sDp_V_REF, float fV_REF,
										string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
										string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  int iAlarm=c_unSystemIntegrity_no_alarm_value;
  string sSTATE_PCLetter, sSTATE_OPLetter, sSTATE_PCColor, sSTATE_OPColor, sOkColor, sBadColor;
  bool bInvalidData;
	
	if(g_bSystemConnected)
		{	
		 if( (iFESystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
			{
			iAlarm = c_unSystemIntegrity_alarm_value_level1;
			bInvalidData = TRUE;
			}
		
		bInvalidData = bInvalidData | bInvalid;
		
		if (bInvalidData)
			{
			sOkColor = "unDataNotValid";
			sBadColor = "unDataNotValid";		
			}
		else
			{
			sOkColor = "unFaceplate_OperationModeActive";
			sBadColor = "unFaceplate_OperationModeActive";
			}
			
		// get STATE_PC, STATE_OP
	  sSTATE_PCLetter = sSTATE_PC;
	  sSTATE_PCColor = _pcFGC_GENERIC_PC_getStatePcColor(bInvalidData, sSTATE_PC);
		
			
    sSTATE_OPLetter = sSTATE_OP;
		if(bInvalidData)
    {
			sSTATE_OPColor = "unDataNotValid";
    }
		else
    {
			sSTATE_OPColor = "unDisplayValue_Status";
    }


		setMultiValue(	
            "STATE_PC", "text", cutText(sSTATE_PCLetter, 10), 
            "STATE_PC", "toolTipText", sSTATE_PCLetter, 
						"STATE_PC", "foreCol", sSTATE_PCColor,
						"STATE_PC", "backCol", "unSynopticBackground",
						"STATE_OP", "text", cutText(sSTATE_OPLetter, 10), 
            "STATE_OP", "toolTipText", sSTATE_OPLetter,
						"STATE_OP", "foreCol", sSTATE_OPColor,
						"STATE_OP", "backCol", "unSynopticBackground"
            );
		
		unGenericObject_DisplayValue(g_sIFormat, g_sIUnit, fI_MEAS, "I_MEAS", "unDisplayValue_Status", bInvalidData);
		unGenericObject_DisplayValue(g_sVFormat, g_sVUnit, fV_MEAS, "V_MEAS", "unDisplayValue_Status", bInvalidData);
		unGenericObject_DisplayValue(g_sIFormat, g_sIUnit, fI_REF, "I_REF", "unDisplayValue_Status", bInvalidData);
		unGenericObject_DisplayValue(g_sVFormat, g_sVUnit, fV_REF, "V_REF", "unDisplayValue_Status", bInvalidData);
		
		pcFGC_GENERIC_PC_FaceplateStatus_Header(g_sFrontEndDp, iAlarm, bFESystemIntegrityAlarmEnabled, bInvalidData);
	
		
		//ST_FAULTS
    	dyn_string dsST_FAULTS = strsplit(sST_FAULTS, " "); 
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_FGC_HW", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "FGC_HW"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_I_MEAS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "I_MEAS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_REG_ERROR", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "REG_ERROR"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_POL_SWITCH", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "POL_SWITCH"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_NO_PC_PERMIT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "NO_PC_PERMIT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_STATE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "VS_STATE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_RUN_TO", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "VS_RUN_TO"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_FAULT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "VS_FAULT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_EXTINTLOCK", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "VS_EXTINTLOCK"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_FGC_STATE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "FGC_STATE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_FAST_ABORT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "FAST_ABORT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_LIMITS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "LIMITS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_SLAVE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "SLAVE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_VS_COMMS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "VS_COMMS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_V_ERROR", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "V_ERROR"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_FABORT_UNSAFE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "FABORT_UNSAFE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_FAULTS_FW_DIODE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_FAULTS, "FW_DIODE"), true, sOkColor, sBadColor, bInvalidData);
		
		
    //ST_UNLATCHED
    dyn_string dsST_UNLATCHED = strsplit(sST_UNLATCHED, " ");
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_NOMINAL_LOAD", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "NOMINAL_LOAD"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_SYNC_PLEASE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "SYNC_PLEASE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_I_MEAS_DIFF", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "I_MEAS_DIFF"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_POL_SWI_POS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "POL_SWI_POS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_POL_SWI_NEG", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "POL_SWI_NEG"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_PWR_FAILURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PWR_FAILURE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_PC_DISCH_RQ", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_DISCH_RQ"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_PC_PERMIT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "PC_PERMIT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_POST_MORTEM", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "POST_MORTEM"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_REF_RT_ACTIVE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "REF_RT_ACTIVE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_VS_POWER_ON", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "VS_POWER_ON"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_NO_VS_CABLE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "NO_VS_CABLE"), true, sOkColor, sBadColor, bInvalidData);
    unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_LOW_CURRENT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "LOW_CURRENT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_START_EVENT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "START_EVENT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_UNLATCHED_ABORT_EVENT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState(dsST_UNLATCHED, "ABORT_EVENT"), true, sOkColor, sBadColor, bInvalidData);    
    
		//ST_WARNINGS
		dyn_string dsST_WARNINGS = strsplit(sST_WARNINGS, " ");
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_FGC_HW", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "FGC_HW"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_I_MEAS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "I_MEAS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_REG_ERROR", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "REG_ERROR"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_ILIM_EXPECTED", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "ILIM_EXPECTED"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_FGC_PSU", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "FGC_PSU"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_TEMPERATURE", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "TEMPERATURE"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_SUBCONVTR_FLT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "SUBCONVTR_FLT"), true, sOkColor, sBadColor, bInvalidData);
    unGenericObject_ColoredSquareAnimate("ST_WARNINGS_REF_LIM", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "REF_LIM"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_REF_RATE_LIM", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "REF_RATE_LIM"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_V_ERROR", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "V_ERROR"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_SIMULATION", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "SIMULATION"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_CONFIG", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "CONFIG"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_I_RMS_LIM", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "I_RMS_LIM"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_TIMING_EVT", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "TIMING_EVT"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_CYCLE_START", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "CYCLE_START"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_VS_COMMS", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "VS_COMMS"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_DCCT_PSU", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "DCCT_PSU"), true, sOkColor, sBadColor, bInvalidData);
		unGenericObject_ColoredSquareAnimate("ST_WARNINGS_CURRENT_LEAD", pcFGC_GENERIC_PC_FaceplateStatus_getBitState( dsST_WARNINGS, "CURRENT_LEAD"), true, sOkColor, sBadColor, bInvalidData);
    
    

  		}
}

//---------------------------------------------------------------------------------------------------------------------------------------
//  ??? not used anymore
// pcFGC_GENERIC_PC_FaceplateStatus_SetUnknown
/**
Purpose: set the grey color for the dsSatusShape, ?? for the dsValueShape

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateStatus_SetUnknown(bool bConnected, dyn_string dsSatusShape, dyn_string dsValueShape)
{
	int i, length;
	
	length = dynlen(dsSatusShape);
	for(i=1;i<=length;i++)
	{
		if(bConnected)
			setValue(dsSatusShape[i] + ".border", "backCol", "unDataNotUpToDate");
	}

	length = dynlen(dsValueShape);
	for(i=1;i<=length;i++)
	{
		if(bConnected)
			setMultiValue(dsValueShape[i] + ".display", "text", PC_FGC_UNKNOWN_VALUE, dsValueShape[i] + ".display", "foreCol", "unDataNotUpToDate");
	}
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateStatus_getBitState
/**
Purpose: This function check if the key is present in the list dsList

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
bool pcFGC_GENERIC_PC_FaceplateStatus_getBitState(const dyn_string &dsList, string sKey)
{
  return dynContains( dsList, sKey) == 0 ? FALSE : TRUE;
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateStatus_Header
/**
Purpose: This function set the text and the colors for the header in FacePlate (state of the PLC connection)

Parameters:

Usage: Internal

PVSS manager usage: NG, NV

Constraints:
	. Global variables defined in pcFGC_GENERIC faceplate
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
bool pcFGC_GENERIC_PC_FaceplateStatus_Header(const string sDp, const int iAlarm, const bool bSystemAlarmEnable, const bool bInvalid=FALSE)
{
bool bOk=false, bFrontEnd=false;
time tLast;
string sBackColor, sForeColor, sText, sTime, sName;



	sName=unGenericDpFunctions_getAlias(sDp);
	if (sName=="")
		sName="?";
	else if (sName!=sDp)
		bFrontEnd=true;
	
	
	if (bInvalid)
		{
		sBackColor="unDataNotValid";
		sForeColor="_3DText";
		sTime="";
		sText="DATA INVALID";
		}
	else if (!bSystemAlarmEnable)
		{
		//SystemAlarm disabled
		sBackColor="unAlarmMasked";
		sForeColor="_3DText";
		sTime="";
		sText=sName + " DATA Connection DISABLED";
		}
	else
		{
		if (iAlarm > c_unSystemIntegrity_no_alarm_value)
			{
			//Connection stopped
			sBackColor="unDataNotValid";
			sForeColor="_3DText";
			sText=sName + " NO DATA Connection";
			sTime="";
			}
		else
			{
			//Connection OK
			sBackColor="_3DFace";
			sForeColor="_3DText";
			sText=sName + " DATA Connection OK";
			sTime="";			
			bOk=true;
			}
		}
	
	sText=sText + sTime;
	if(shapeExists("txtPLCstatus"))
		{
		setMultiValue("txtPLCstatus", "backCol", sBackColor,
									"txtPLCstatus", "foreCol", sForeColor,
									"txtPLCstatus", "text", sText);
		}
	else
		DebugN("pcFGC_GENERIC_PC_FaceplateStatus_Header", "\"txtPLCstatus\" shape doesn't exist! Contact UNICOS support");

return bOk;		
}

//---------------------------------------------------------------------------------------------------------------------------------------

// pcFGC_GENERIC_PC_FaceplateStatusDisconnection
/**
Purpose: Animate faceplate disconnection

Parameters: None

Usage: External

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.0 
	. operating system: NT and W2000, but tested only under W2000.
	. distributed system: yes.
*/
pcFGC_GENERIC_PC_FaceplateStatusDisconnection()
{
// 1. animate color boxes

	unGenericObject_ColoredSquareDisconnect(makeDynString("ST_UNLATCHED_NOMINAL_LOAD", "ST_UNLATCHED_SYNC_PLEASE",
																												"ST_UNLATCHED_I_MEAS_DIFF", "ST_UNLATCHED_POL_SWI_POS",
																												"ST_UNLATCHED_POL_SWI_NEG", "ST_UNLATCHED_PWR_FAILURE",
																												"ST_UNLATCHED_PC_DISCH_RQ", "ST_UNLATCHED_PC_PERMIT",
																												"ST_UNLATCHED_POST_MORTEM", "ST_UNLATCHED_REF_RT_ACTIVE",
																												"ST_UNLATCHED_VS_POWER_ON",
																												"ST_UNLATCHED_NO_VS_CABLE", "ST_UNLATCHED_LOW_CURRENT",
																												"ST_UNLATCHED_START_EVENT", "ST_UNLATCHED_ABORT_EVENT",
																												"ST_FAULTS_FGC_HW", "ST_FAULTS_I_MEAS",
																												"ST_FAULTS_REG_ERROR", "ST_FAULTS_POL_SWITCH",
																												"ST_FAULTS_NO_PC_PERMIT", "ST_FAULTS_VS_STATE",
																												"ST_FAULTS_VS_RUN_TO", "ST_FAULTS_VS_FAULT",
																												"ST_FAULTS_VS_EXTINTLOCK", "ST_FAULTS_FGC_STATE",
																												"ST_FAULTS_FAST_ABORT", "ST_FAULTS_LIMITS",
																												"ST_FAULTS_FW_DIODE", "ST_FAULTS_FABORT_UNSAFE", "ST_FAULTS_V_ERROR",
																												"ST_WARNINGS_FGC_HW", "ST_WARNINGS_I_MEAS",
                                                        "ST_WARNINGS_REG_ERROR", "ST_WARNINGS_ILIM_EXPECTED",
																												"ST_WARNINGS_FGC_PSU", "ST_WARNINGS_TEMPERATURE",
                                                        "ST_WARNINGS_SUBCONVTR_FLT", "ST_WARNINGS_DCCT_PSU",
																												"ST_WARNINGS_REF_LIM",
																												"ST_WARNINGS_REF_RATE_LIM", "ST_WARNINGS_V_ERROR",
																												"ST_WARNINGS_SIMULATION", "ST_WARNINGS_CONFIG", "ST_WARNINGS_CURRENT_LEAD"));
							 
	// 2. animate Current and Voltage
	unGenericObject_DisplayValueDisconnect(makeDynString("I_MEAS", "V_MEAS", "I_REF", "V_REF"));
	
	setMultiValue("STATE_PC", "color", "unDataNoAccess",
								"STATE_OP", "color", "unDataNoAccess");
								
// 2. animate PLC Connection
	pcFGC_GENERIC_PC_FaceplateStatus_HeaderDisconnect();
}

//------------------------------------------------------------------------------------------------------------------------
// pcFGC_GENERIC_PC_FaceplateStatus_HeaderDisconnect
/**
Purpose: update header to "Disconnect look and feel"

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
void pcFGC_GENERIC_PC_FaceplateStatus_HeaderDisconnect()
{
	string sBackColor, sForeColor, sText;

	sBackColor="_3DFace";
	sForeColor="unDataNoAccess";
	sText="DATA not connected";
				
	if(shapeExists("txtPLCstatus"))
		{
		setMultiValue("txtPLCstatus", "backCol", sBackColor,
									"txtPLCstatus", "foreCol", sForeColor,
									"txtPLCstatus", "text", sText);
		}
	else
		DebugN("pcFGC_GENERIC_PC_FaceplateStatus_HeaderDisconnect", "\"txtPLCstatus\" shape doesn't exist! Contact UNICOS support");
}

//------------------------------------------------------------------------------------------------------------------------
// pcFGC_GENERIC_PC_setFGCValue
/**
Purpose: update header to "Disconnect look and feel"

Parameters:

Usage: External function

PVSS manager usage: NG, NV
*/
pcFGC_GENERIC_PC_setFGCValue(string sDpe, anytype aValue, dyn_string &exInfo)
{
  string sValue;
  int iValue;
  dyn_int diValue;
  
  switch(getType(aValue)) {
    case STRING_VAR:
      sValue = (string)aValue;
      dpSetWait(sDpe, sValue);
      break;
    case INT_VAR:
      iValue = (int)aValue;
      dpSetWait(sDpe, iValue);
      break;
    case DYN_INT_VAR:
      diValue = (dyn_int)diValue;
      dpSetWait(sDpe, diValue);
      break;
  }
}


/**
  Given the Invalid flag and the string content of the STATE_PC,
  returns the color used to display the STATE_PC in widget, faceplate, etc.
  Spec. is coming mostly form ENS-21559
  **/
private string _pcFGC_GENERIC_PC_getStatePcColor( bool bInvalidData, string sSTATE_PC)
{
  string sSTATE_PCColor;
  if (bInvalidData)
  {
    sSTATE_PCColor = "unDataNotValid";   
  }
  else if ( strpos(sSTATE_PC, PC_FGC_STATE_FLT_OFF) > -1) //ENS-21559, text in red if FLT_OFF
  {
    sSTATE_PCColor = "unFaceplate_AlarmActive";
  } else
  {
			sSTATE_PCColor = "unDisplayValue_Status";
  }
  
  return sSTATE_PCColor;
}
